--[[
    Broker_RecZone by LordFarlander
    Version 3.0.18
    $Revision: 17 $

    A conversion of Fubar_RecZone by wmrojer
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Broker_RecZone" );
local qtip = LibStub( "LibQTip-1.0" );
local Jostle = LibStub( "LibJostle-3.0" );

Broker_RecZone = LibStub( "AceAddon-3.0" ):NewAddon( "Broker_RecZone", "AceEvent-3.0", "AceConsole-3.0", "AceTimer-3.0" );

Broker_RecZone.version = "3.0." .. "18";
Broker_RecZone.date = ("$Date: 2008-12-04 03:51:22 +0000 (Thu, 04 Dec 2008) $"):sub( 8, 17 );

local clickProvider, clickPrototype, baseclickCellPrototype = qtip:CreateCellProvider( qtip.LabelProvider );

function clickPrototype:InitializeCell()
    baseclickCellPrototype.InitializeCell( self );
end--clickPrototype:InitializeCell()

function clickPrototype:SetupCell( tooltip, value, justification, font, class, zone )
    self:EnableMouse( true );
    self:SetScript( "OnMouseDown", function() class:OnClickZone( zone ) end );
    return baseclickCellPrototype.SetupCell( self, tooltip, value, justification, font );
end--clickPrototype:SetupCell( tooltip, value, justification, font, class, zone )

-- Enumerations.
-- Faction
local RZ_ALLIANCE      = 1;
local RZ_HORDE         = 2;
local RZ_SANCTUARY     = 3;
local RZ_CONTESTED     = 4;
-- Continent
local RZ_KALIMDOR      = 1;
local RZ_EASTERN       = 2;
local RZ_OUTLAND       = 3;
local RZ_NORTHREND     = 4;
local RZ_OTHER         = 255;
-- Type
local RZ_ZONE          = 1;
local RZ_CITY          = 2;
local RZ_TRANSPORT     = 3;
local RZ_NONE          = 4;
local RZ_ARENA         = 5;
local RZ_INSTANCE      = 6;
local RZ_RAID          = 7;
local RZ_BATTLEGROUND  = 8;
local RZ_COLLECTOR     = 9;
local RZ_PVP           = 255;

local RZ_MAXLEVEL      = 80; -- Is there a way to get this programatically?

do
    local Z = LibStub( "LibBabble-Zone-3.0" ):GetLookupTable();

    Broker_RecZone.vars = {
        current_continent = nil,
        current_zone = nil,
        playerLevel = 0,
        zoneName = "",
        subZoneName = "",
        playerFaction = 0,
        zonetext = "",
        coordtext = "",
        leveltext = "",
        playerCoordX = 0,
        playerCoordY = 0,
        factions = {
            [RZ_ALLIANCE]     = FACTION_ALLIANCE,
            [RZ_HORDE]        = FACTION_HORDE,
            [RZ_CONTESTED]    = L["Contested"],
            [RZ_SANCTUARY]    = L["Sanctuary"],
            [RZ_PVP]          = L["PVP"],
        },
        types = {
            [RZ_ZONE]         = "",
            [RZ_CITY]         = "City",
            [RZ_TRANSPORT]    = "Transport",
            [RZ_NONE]         = "",
            [RZ_INSTANCE]     = "",
            [RZ_BATTLEGROUND] = L["Battleground"],
            [RZ_ARENA]        = L["Arena"],
            [RZ_PVP]          = L["PVP"],
            [RZ_RAID]         = "",
            [RZ_COLLECTOR]    = "",
        },
        continents = {},
        zones = {
    -- Eastern
            { name = Z["Stormwind City"], low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_CITY, continent = RZ_EASTERN },
            { name = Z["Ironforge"], low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_CITY, continent = RZ_EASTERN },
            { name = Z["Undercity"], low = 0, high = 0, faction = RZ_HORDE, type = RZ_CITY, continent = RZ_EASTERN },
            { name = Z["Dun Morogh"], low = 1, high = 12, faction = RZ_ALLIANCE, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Elwynn Forest"], low = 1, high = 12, faction = RZ_ALLIANCE, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Tirisfal Glades"], low = 1, high = 12, faction = RZ_HORDE, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Loch Modan"], low = 10, high = 20, faction = RZ_ALLIANCE, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Westfall"], low = 10, high = 20, faction = RZ_ALLIANCE, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Silverpine Forest"], low = 10, high = 20, faction = RZ_HORDE, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Redridge Mountains"], low = 15, high = 25, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Wetlands"], low = 20, high = 30, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Duskwood"], low = 20, high = 30, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Hillsbrad Foothills"], low = 20, high = 30, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Alterac Mountains"], low = 30, high = 40, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Arathi Highlands"], low = 30, high = 40, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Stranglethorn Vale"], low = 30, high = 45, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Badlands"], low = 35, high = 45, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Swamp of Sorrows"], low = 35, high = 45, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["The Hinterlands"], low = 40, high = 50, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Searing Gorge"], low = 43, high = 50, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Blasted Lands"], low = 47, high = 55, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Burning Steppes"], low = 50, high = 60, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Western Plaguelands"], low = 50, high = 60, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Deadwind Pass"], low = 55, high = 60, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Eastern Plaguelands"], low = 55, high = 60, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Isle of Quel'Danas"], low = 65, high = 70, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },

    -- Kalimdor
            { name = Z["Darnassus"], low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Orgrimmar"], low = 0, high = 0, faction = RZ_HORDE, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Thunder Bluff"], low = 0, high = 0, faction = RZ_HORDE, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Moonglade"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Teldrassil"], low = 1, high = 12, faction = RZ_ALLIANCE, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Durotar"], low = 1, high = 12, faction = RZ_HORDE, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Mulgore"], low = 1, high = 12, faction = RZ_HORDE, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Darkshore"], low = 10, high = 20, faction = RZ_ALLIANCE, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["The Barrens"], low = 10, high = 25, faction = RZ_HORDE, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Stonetalon Mountains"], low = 15, high = 27, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Ashenvale"], low = 15, high = 30, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Thousand Needles"], low = 25, high = 35, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Desolace"], low = 30, high = 40, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Dustwallow Marsh"], low = 35, high = 45, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Feralas"], low = 40, high = 50, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Tanaris"], low = 40, high = 50, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Un'Goro Crater"], low = 48, high = 55, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Azshara"], low = 48, high = 55, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Felwood"], low = 48, high = 55, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Silithus"], low = 55, high = 60, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Winterspring"], low = 55, high = 60, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_KALIMDOR },

    -- Blood Elf
            { name = Z["Silvermoon City"], low = 0, high = 0, faction = RZ_HORDE, type = RZ_CITY, continent = RZ_EASTERN },
            { name = Z["Eversong Woods"], low = 1, high = 12, faction = RZ_HORDE, type = RZ_ZONE, continent = RZ_EASTERN },
            { name = Z["Ghostlands"], low = 10, high = 20, faction = RZ_HORDE, type = RZ_ZONE, continent = RZ_EASTERN },

    -- Draenei
            { name = Z["The Exodar"], low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Azuremyst Isle"], low = 1, high = 12, faction = RZ_ALLIANCE, type = RZ_ZONE, continent = RZ_KALIMDOR },
            { name = Z["Bloodmyst Isle"], low = 10, high = 20, faction = RZ_ALLIANCE, type = RZ_ZONE, continent = RZ_KALIMDOR },

    -- Outland
            { name = Z["Shattrath City"], low = 0, high = 0, faction = RZ_SANCTUARY, type = RZ_CITY, continent = RZ_OUTLAND },
            { name = Z["Hellfire Peninsula"], low = 58, high = 63, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_OUTLAND },
            { name = Z["Zangarmarsh"], low = 60, high = 64, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_OUTLAND },
            { name = Z["Terokkar Forest"], low = 62, high = 65, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_OUTLAND },
            { name = Z["Nagrand"], low = 64, high = 67, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_OUTLAND },
            { name = Z["Blade's Edge Mountains"], low = 65, high = 68, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_OUTLAND },
            { name = Z["Netherstorm"], low = 67, high = 70, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_OUTLAND },
            { name = Z["Shadowmoon Valley"], low = 67, high = 70, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_OUTLAND },

    -- Northrend
            { name = Z["Borean Tundra"], low = 68, high = 72, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["Howling Fjord"], low = 68, high = 72, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["Dragonblight"], low = 71, high = 74, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["Grizzly Hills"], low = 73, high = 75, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["Zul'Drak"], low = 74, high = 77, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["Sholazar Basin"], low = 75, high = 78, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["Crystalsong Forest"], low = 77, high = 80, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["The Storm Peaks"], low = 76, high = 80, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["Icecrown"], low = 77, high = 80, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_NORTHREND },
            { name = Z["Wintergrasp"], low = 77, high = 80, faction = RZ_PVP, type = RZ_PVP, continent = RZ_NORTHREND },
            { name = Z["Dalaran"], low = 0, high = 0, faction = RZ_SANCTUARY, type = RZ_CITY, continent = RZ_NORTHREND },

    -- Death Knights
            { name = Z["Plaguelands: The Scarlet Enclave"], low = 55, high = 58, faction = RZ_CONTESTED, type = RZ_ZONE, continent = RZ_EASTERN },

    -- Instances
            { name = Z["Ragefire Chasm"], low = 13, high = 20, type = RZ_INSTANCE, faction = RZ_HORDE, continent = RZ_KALIMDOR },
            { name = Z["Wailing Caverns"], low = 17, high = 24, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_KALIMDOR },
            { name = Z["The Deadmines"], low = 17, high = 26, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["The Stockade"], low = 24, high = 32, type = RZ_INSTANCE, faction = RZ_ALLIANCE, continent = RZ_EASTERN },
            { name = Z["Shadowfang Keep"], low = 20, high = 28, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Blackfathom Deeps"], low = 23, high = 30, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_KALIMDOR },
            { name = Z["Gnomeregan"], low = 29, high = 38, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Razorfen Kraul"], low = 29, high = 38, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_KALIMDOR },
            { name = Z["Scarlet Monastery"], low = 33, high = 44, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Razorfen Downs"], low = 37, high = 46, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_KALIMDOR },
            { name = Z["Uldaman"], low = 38, high = 46, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Maraudon"], low = 45, high = 55, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_KALIMDOR },
            { name = Z["Zul'Farrak"], low = 43, high = 50, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_KALIMDOR },
            { name = Z["The Temple of Atal'Hakkar"], low = 44, high = 52, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Blackrock Depths"], low = 50, high = 56, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Blackrock Spire"], low = 56, high = 60, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Stratholme"], low = 56, high = 60, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Dire Maul"], low = 56, high = 60, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_KALIMDOR },
            { name = Z["Scholomance"], low = 58, high = 60, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Blackwing Lair"], low = 60, high = 62, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_EASTERN, size = 40 },
            { name = Z["Molten Core"], low = 60, high = 62, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_EASTERN, size = 40 },
            { name = Z["Onyxia's Lair"], low = 60, high = 62, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_KALIMDOR, size = 40 },
            { name = Z["Zul'Gurub"], low = 60, high = 62, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_EASTERN, size = 20 },
            { name = Z["Ahn'Qiraj"], low = 60, high = 62, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_KALIMDOR, size = 20 },
            { name = Z["Temple of Ahn'Qiraj"], low = 60, high = 62, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_EASTERN, size = 40 },
            { name = Z["Warsong Gulch"], x = 1, low = 10, high = -1, type = RZ_BATTLEGROUND, faction = RZ_HORDE, continent = RZ_KALIMDOR },
            { name = Z["Warsong Gulch"], x = 2, low = 10, high = -1, type = RZ_BATTLEGROUND, faction = RZ_ALLIANCE, continent = RZ_KALIMDOR },
            { name = Z["Arathi Basin"], low = 20, high = -1, type = RZ_BATTLEGROUND, faction = RZ_CONTESTED, continent = RZ_EASTERN },
            { name = Z["Alterac Valley"], low = 51, high = -1, type = RZ_BATTLEGROUND, faction = RZ_CONTESTED, continent = RZ_EASTERN },

    -- BC Instances
            { name = Z["Hellfire Ramparts"], low = 60, high = 62, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Blood Furnace"], low = 61, high = 63, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Slave Pens"], low = 62, high = 64, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Underbog"], low = 63, high = 65, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["Auchenai Crypts"], low = 64, high = 66, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["Shadow Labyrinth"], low = 65, high = 67, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["Sethekk Halls"], low = 67, high = 69, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Mechanar"], low = 69, high = 72, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Shattered Halls"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Botanica"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Arcatraz"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Eye"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_OUTLAND, size = 25 },
            { name = Z["Mana-Tombs"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["The Steamvault"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_OUTLAND },
            { name = Z["Magisters' Terrace"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_EASTERN },
            { name = Z["Old Hillsbrad Foothills"], low = 66, high = 68, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_KALIMDOR },
            { name = Z["The Black Morass"], low = 68, high = 70, faction = RZ_CONTESTED, type = RZ_INSTANCE, continent = RZ_KALIMDOR },
            { name = Z["Karazhan"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_EASTERN, size = 10 },
            { name = Z["Gruul's Lair"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_OUTLAND, size = 25 },
            { name = Z["Magtheridon's Lair"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_OUTLAND, size = 25 },
            { name = Z["Black Temple"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_OUTLAND, size = 25 },
            { name = Z["Serpentshrine Cavern"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_OUTLAND, size = 25 },
            { name = Z["Sunwell Plateau"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_EASTERN, size = 25 },
            { name = Z["Zul'Aman"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_EASTERN, size = 10 },
            { name = Z["Hyjal Summit"], low = 70, high = 72, faction = RZ_CONTESTED, type = RZ_RAID, continent = RZ_KALIMDOR, size = 25 },
            { name = Z["Eye of the Storm"], low = 61, high = -1, faction = RZ_CONTESTED, type = RZ_BATTLEGROUND, continent = RZ_OUTLAND },

    -- WotLK Instances
            { name = Z["Utgarde Keep"], x = 2, low = 70, high = 72, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Utgarde Pinnacle"], low = 80, high = 82, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["The Oculus"], low = 80, high = 82, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["The Nexus"], low = 71, high = 73, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Azjol-Nerub"], low = 72, high = 74, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Ahn'kahet: The Old Kingdom"], low = 73, high = 75, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Drak'Tharon Keep"], low = 74, high = 76, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["The Violet Hold"], low = 75, high = 77, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Gundrak"], low = 76, high = 78, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Halls of Stone"], low = 77, high = 79, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Halls of Lightning"], low = 80, high = 82, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
    --        { name = Z["Ulduar Raid"], low = 80, high = 82, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_NORTHREND, size = { 10, 20 } }, --NYI
            { name = Z["The Culling of Stratholme"], low = 80, high = 82, type = RZ_INSTANCE, faction = RZ_CONTESTED, continent = RZ_KALIMDOR },
            { name = Z["The Nexus"], x = 2, low = 80, high = 82, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_NORTHREND, size = { 10, 20 } },
            { name = Z["The Eye of Eternity"], low = 80, high = 82, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_NORTHREND, size = { 10, 20 } },
            { name = Z["The Obsidian Sanctum"], low = 80, high = 82, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_NORTHREND, size = { 10, 20 } },
            { name = Z["Naxxramas"], low = 80, high = 82, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_NORTHREND, size = { 10, 20 } },
            { name = Z["Icecrown Citadel"], low = 80, high = 82, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_NORTHREND, size = { 10, 20 }, NYI = true },
            { name = L["Vault of Archavon"], low = 80, high = 82, type = RZ_RAID, faction = RZ_CONTESTED, continent = RZ_NORTHREND, size = { 10, 20 } },
            { name = Z["Strand of the Ancients"], low = 71, high = -1, type = RZ_BATTLEGROUND, faction = RZ_CONTESTED, continent = RZ_NORTHREND },

    -- Towns and transport zones
            { name = Z["Booty Bay"], low = 30, high = 45, faction = RZ_CONTESTED, type = RZ_CITY, continent = RZ_EASTERN },
            { name = Z["Ratchet"], low = 10, high = 25, faction = RZ_CONTESTED, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Gadgetzan"], low = 40, high = 50, faction = RZ_CONTESTED, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Deeprun Tram"], low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_CITY, continent = RZ_EASTERN },
            { name = Z["Theramore Isle"], low = 35, high = 45, faction = RZ_ALLIANCE, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Menethil Harbor"], low = 20, high = 30, faction = RZ_ALLIANCE, type = RZ_CITY, continent = RZ_EASTERN },
            { name = Z["Auberdine"], low = 10, high = 20, faction = RZ_ALLIANCE, type = RZ_CITY, continent = RZ_KALIMDOR },
            { name = Z["Grom'gol Base Camp"], low = 30, high = 45, faction = RZ_HORDE, type = RZ_CITY, continent = RZ_EASTERN },
            { name = Z["Blackrock Mountain"], low = 60, high = 62, type = RZ_COLLECTOR, faction = RZ_CONTESTED, continent = RZ_EASTERN },

            { name = Z["Caverns of Time"], low = 66, high = 82, faction = RZ_CONTESTED, type = RZ_COLLECTOR, continent = RZ_KALIMDOR },
            { name = Z["Hellfire Citadel"], low = 60, high = 72, faction = RZ_CONTESTED, type = RZ_COLLECTOR, continent = RZ_OUTLAND },
            { name = Z["The Bone Wastes"], low = 64, high = 72, faction = RZ_CONTESTED, type = RZ_COLLECTOR, continent = RZ_OUTLAND },
            { name = Z["Coilfang Reservoir"], low = 62, high = 72, faction = RZ_CONTESTED, type = RZ_COLLECTOR, continent = RZ_OUTLAND },
            { name = Z["Tempest Keep"], low = 64, high = 72, faction = RZ_CONTESTED, type = RZ_COLLECTOR, continent = RZ_OUTLAND },
            { name = Z["Coldarra"], low = 71, high = 82, type = RZ_COLLECTOR, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Ulduar"], low = 77, high = 82, type = RZ_COLLECTOR, faction = RZ_CONTESTED, continent = RZ_NORTHREND },
            { name = Z["Utgarde Keep"], x = 1, low = 70, high = 82, type = RZ_COLLECTOR, faction = RZ_CONTESTED, continent = RZ_NORTHREND },

    -- Transports
            { name = L["Boat"], x = 1, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Boat"], x = 2, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Boat"], x = 3, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Boat"], x = 4, low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Boat"], x = 5, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Boat"], x = 6, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Boat"], x = 7, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Boat"], x = 8, low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Boat"], x = 9, low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 1, low = 70, high = -1, faction = RZ_CONTESTED, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 2, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 3, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 4, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 5, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 6, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 7, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 8, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 9, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 10, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 11, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 12, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 13, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 14, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 15, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 16, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 17, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 18, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 19, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 20, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 21, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 22, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 23, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 24, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 25, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 26, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 27, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 28, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 29, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 30, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 31, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 32, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 33, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 34, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Portal"], x = 35, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Zeppelin"], x = 1, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Zeppelin"], x = 2, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Zeppelin"], x = 3, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Zeppelin"], x = 4, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Zeppelin"], x = 5, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },

    -- Battlemasters
            { name = L["Battlemaster"], x = 1, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 2, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 3, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 4, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 5, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 6, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 7, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 8, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 9, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 10, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 11, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 12, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 13, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 14, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 15, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 16, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 17, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 18, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 19, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 20, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 21, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 22, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 23, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 24, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 25, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 26, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 27, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 28, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 29, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 30, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 31, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 32, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 33, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 34, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 35, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 36, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 37, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 38, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 39, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 40, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 41, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 42, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 43, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 44, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 45, low = 0, high = 0, faction = RZ_ALLIANCE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 46, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 47, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 48, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 49, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Battlemaster"], x = 50, low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },

    -- Transport between old and new world
            { name = Z["The Dark Portal"], low = 58, high = -1, faction = RZ_CONTESTED, type = RZ_TRANSPORT, continent = RZ_OTHER },
            { name = L["Orb of Translocation"], low = 0, high = 0, faction = RZ_HORDE, type = RZ_TRANSPORT, continent = RZ_OTHER },

    -- Not real zones
            { name = Z["The Forbidding Sea"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_NONE, continent = RZ_EASTERN },
            { name = Z["Twisting Nether"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_NONE, continent = RZ_OUTLAND },
            { name = Z["The Great Sea"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_NONE, continent = RZ_OTHER },
            { name = Z["The North Sea"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_NONE, continent = RZ_EASTERN },
            { name = Z["The Veiled Sea"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_NONE, continent = RZ_KALIMDOR },
            { name = Z["Hyjal"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_NONE, continent = RZ_KALIMDOR },
            { name = Z["The Frozen Sea"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_NONE, continent = RZ_NORTHREND },

    -- Arenas
            { name = Z["Blade's Edge Arena"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_ARENA, continent = RZ_OUTLAND },
            { name = Z["Nagrand Arena"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_ARENA, continent = RZ_OUTLAND },
            { name = Z["Ruins of Lordaeron"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_ARENA, continent = RZ_EASTERN },
            { name = Z["Dalaran Sewers"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_ARENA, continent = RZ_NORTHREND },
            { name = Z["The Ring of Valor"], low = 0, high = 0, faction = RZ_CONTESTED, type = RZ_ARENA, continent = RZ_KALIMDOR },
        },
    };
    
    local zones = Broker_RecZone.vars.zones;

    local function zn( zonename, x )
        for nr, zone in ipairs( zones ) do
            if( ( zone.name == zonename ) and ( ( not x ) or ( zone.x and ( zone.x == x ) ) ) ) then
                return nr;
            end--if
        end--for
    end--function zn( zonename, x )

    table.sort( Broker_RecZone.vars.zones,
        function( a, b )
            if( a.high == -1 ) then
                a.high = RZ_MAXLEVEL
            end--if
            if( b.high == -1 ) then
                b.high = RZ_MAXLEVEL
            end--if
            return ( a.low < b.low ) or ( ( a.low == b.low ) and ( a.high < b.high ) ) or ( ( a.low == b.low ) and ( a.high == b.high ) and ( a.name < b.name ) );
        end );

    zones[zn( Z["Stormwind City"] )].zones = { zn( Z["Deeprun Tram"] ), zn( Z["Elwynn Forest"] ), zn( Z["The Stockade"] ), zn( L["Boat"], 2 ), zn( L["Boat"], 6 ), zn( L["Battlemaster"], 1 ), zn( L["Battlemaster"], 2 ), zn( L["Battlemaster"], 3 ), zn( L["Battlemaster"], 4 ), zn( L["Battlemaster"], 5 ) };
    zones[zn( Z["Ironforge"] )].zones = { zn( Z["Deeprun Tram"] ), zn( Z["Dun Morogh"] ), zn( Z["Isle of Quel'Danas"] ), zn( L["Portal"], 2 ), zn( L["Battlemaster"], 6 ), zn( L["Battlemaster"], 7 ), zn( L["Battlemaster"], 8 ), zn( L["Battlemaster"], 9 ), zn( L["Battlemaster"], 10 ) };
    zones[zn( Z["Darnassus"] )].zones = { zn( Z["Teldrassil"] ), zn( L["Boat"], 3 ), zn( L["Battlemaster"], 11 ), zn( L["Battlemaster"], 12 ), zn( L["Battlemaster"], 13 ), zn( L["Battlemaster"], 14 ), zn( L["Battlemaster"], 15 ) };
    zones[zn( Z["Undercity"] )].zones = { zn(L["Orb of Translocation"] ), zn( Z["Tirisfal Glades"] ), zn( L["Battlemaster"], 21 ), zn( L["Battlemaster"], 22 ), zn( L["Battlemaster"], 23 ), zn( L["Battlemaster"], 24 ), zn( L["Battlemaster"], 25 ) };
    zones[zn( Z["Orgrimmar"] )].zones = { zn( Z["Durotar"] ), zn( Z["Ragefire Chasm"] ), zn( L["Battlemaster"], 26 ), zn( L["Battlemaster"], 27 ), zn( L["Battlemaster"], 28 ), zn( L["Battlemaster"], 29 ), zn( L["Battlemaster"], 30 ) };
    zones[zn( Z["Thunder Bluff"] )].zones = { zn( Z["Mulgore"] ), zn( L["Battlemaster"], 31 ), zn( L["Battlemaster"], 32 ), zn( L["Battlemaster"], 33 ), zn( L["Battlemaster"], 34 ), zn( L["Battlemaster"], 35 ) };
    zones[zn( Z["Moonglade"] )].zones = { zn( Z["Winterspring"] ), zn( Z["Felwood"] ) };
    zones[zn( Z["Dun Morogh"] )].zones = { zn( Z["Ironforge"] ), zn( Z["Loch Modan"] ), zn( Z["Gnomeregan"] ) };
    zones[zn( Z["Elwynn Forest"] )].zones = { zn( Z["Stormwind City"] ), zn( Z["Westfall"] ), zn( Z["Redridge Mountains"] ), zn( Z["Duskwood"] ) };
    zones[zn( Z["Teldrassil"] )].zones = { zn( Z["Darnassus"] ) };
    zones[zn( Z["Tirisfal Glades"] )].zones = { zn( Z["Undercity"] ), zn( Z["Silverpine Forest"] ), zn( Z["Western Plaguelands"] ), zn( L["Zeppelin"], 1 ), zn( L["Zeppelin"], 2 ), zn( Z["Scarlet Monastery"] ), zn( L["Zeppelin"], 5 ) };
    zones[zn( Z["Durotar"] )].zones = { zn( Z["Orgrimmar"] ), zn( Z["The Barrens"] ), zn( Z["Azshara"] ), zn( L["Zeppelin"], 1 ), zn( L["Zeppelin"], 3 ), zn( L["Zeppelin"], 4 ) };
    zones[zn( Z["Mulgore"] )].zones = { zn( Z["Thunder Bluff"] ), zn( Z["The Barrens"] ) };
    zones[zn( Z["Loch Modan"] )].zones = { zn( Z["Dun Morogh"] ), zn( Z["Wetlands"] ), zn( Z["Badlands"] ) };
    zones[zn( Z["Westfall"] )].zones = { zn( Z["Elwynn Forest"] ), zn( Z["Duskwood"] ), zn( Z["The Deadmines"] ) };
    zones[zn( Z["Silverpine Forest"] )].zones = { zn( Z["Tirisfal Glades"] ), zn( Z["Hillsbrad Foothills"] ), zn( Z["Alterac Mountains"] ), zn( Z["Shadowfang Keep"] ) };
    zones[zn( Z["Darkshore"] )].zones = { zn( Z["Ashenvale"] ), zn( Z["Auberdine"] ) };
    zones[zn( Z["The Barrens"] )].zones = { zn( Z["Ratchet"] ), zn( Z["Durotar"] ), zn( Z["Mulgore"] ), zn( Z["Stonetalon Mountains"] ), zn( Z["Ashenvale"] ), zn( Z["Thousand Needles"] ), zn( Z["Dustwallow Marsh"] ), zn( Z["Wailing Caverns"] ), zn( Z["Razorfen Kraul"] ), zn( Z["Razorfen Downs"] ), zn( Z["Warsong Gulch"], 1 ) };
    zones[zn( Z["Redridge Mountains"] )].zones = { zn( Z["Elwynn Forest"] ), zn( Z["Burning Steppes"] ) };
    zones[zn( Z["Stonetalon Mountains"] )].zones = { zn( Z["The Barrens"] ), zn( Z["Ashenvale"] ), zn( Z["Desolace"] ) };
    zones[zn( Z["Ashenvale"] )].zones = { zn( Z["Darkshore"] ), zn( Z["The Barrens"] ), zn( Z["Stonetalon Mountains"] ), zn( Z["Azshara"] ), zn( Z["Felwood"] ), zn( Z["Blackfathom Deeps"] ), zn( Z["Warsong Gulch"], 2 ) };
    zones[zn( Z["Wetlands"] )].zones = { zn( Z["Loch Modan"] ), zn( Z["Arathi Highlands"] ), zn( Z["Menethil Harbor"] ) };
    zones[zn( Z["Duskwood"] )].zones = { zn( Z["Elwynn Forest"] ), zn( Z["Westfall"] ), zn( Z["Stranglethorn Vale"] ), zn( Z["Deadwind Pass"] ) };
    zones[zn( Z["Hillsbrad Foothills"] )].zones = { zn( Z["Silverpine Forest"] ), zn( Z["Alterac Mountains"] ), zn( Z["Arathi Highlands"] ) };
    zones[zn( Z["Thousand Needles"] )].zones = { zn( Z["The Barrens"] ), zn( Z["Feralas"] ), zn( Z["Tanaris"] ) };
    zones[zn( Z["Alterac Mountains"] )].zones = { zn( Z["Silverpine Forest"] ), zn( Z["Hillsbrad Foothills"] ), zn( Z["The Hinterlands"] ), zn( Z["Western Plaguelands"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( Z["Arathi Highlands"] )].zones = { zn( Z["Wetlands"] ), zn( Z["Hillsbrad Foothills"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( Z["Desolace"] )].zones = { zn( Z["Stonetalon Mountains"] ), zn( Z["Feralas"] ), zn( Z["Maraudon"] ) };
    zones[zn( Z["Stranglethorn Vale"] )].zones = { zn( Z["Booty Bay"] ), zn( Z["Duskwood"] ), zn( Z["Grom'gol Base Camp"] ), zn( Z["Zul'Gurub"] ) };
    zones[zn( Z["Badlands"] )].zones = { zn( Z["Loch Modan"] ), zn( Z["Searing Gorge"] ), zn( Z["Uldaman"] ) };
    zones[zn( Z["Swamp of Sorrows"] )].zones = { zn( Z["Blasted Lands"] ), zn( Z["Deadwind Pass"] ), zn( Z["The Temple of Atal'Hakkar"] ) };
    zones[zn( Z["Dustwallow Marsh"] )].zones = { zn( Z["The Barrens"] ), zn( Z["Theramore Isle"] ), zn( Z["Onyxia's Lair"] ) };
    zones[zn( Z["The Hinterlands"] )].zones = { zn( Z["Alterac Mountains"] ), zn( Z["Western Plaguelands"] ) };
    zones[zn( Z["Feralas"] )].zones = { zn( Z["Thousand Needles"] ), zn( Z["Desolace"] ), zn( Z["Dire Maul"] ) };
    zones[zn( Z["Tanaris"] )].zones = { zn( Z["Thousand Needles"] ), zn( Z["Un'Goro Crater"] ), zn( Z["Gadgetzan"] ), zn( Z["Zul'Farrak"] ), zn( Z["Caverns of Time"] ) };
    zones[zn( Z["Searing Gorge"] )].zones = { zn( Z["Badlands"] ), zn( Z["Blackrock Mountain"] ) };
    zones[zn( Z["Blasted Lands"] )].zones = { zn( Z["Swamp of Sorrows"] ), zn( Z["The Dark Portal"] ) };
    zones[zn( Z["Un'Goro Crater"] )].zones = { zn( Z["Tanaris"] ), zn( Z["Silithus"] ) };
    zones[zn( Z["Azshara"] )].zones = { zn( Z["Durotar"] ), zn( Z["Ashenvale"] ) };
    zones[zn( Z["Felwood"] )].zones = { zn( Z["Darkshore"] ), zn( Z["Ashenvale"] ), zn( Z["Winterspring"] ), zn( Z["Moonglade"] ) };
    zones[zn( Z["Burning Steppes"] )].zones = { zn( Z["Redridge Mountains"] ), zn( Z["Blackrock Mountain"] ) };
    zones[zn( Z["Western Plaguelands"] )].zones = { zn( Z["Tirisfal Glades"] ), zn( Z["Alterac Mountains"] ), zn( Z["The Hinterlands"] ), zn( Z["Eastern Plaguelands"] ), zn( Z["Scholomance"] ) };
    zones[zn( Z["Deadwind Pass"] )].zones = { zn( Z["Duskwood"] ), zn( Z["Swamp of Sorrows"] ), zn( Z["Karazhan"] ) };
    zones[zn( Z["Karazhan"] )].zones = { zn( Z["Deadwind Pass"] ) };
    zones[zn( Z["Eastern Plaguelands"] )].zones = { zn( Z["Western Plaguelands"] ), zn( Z["Stratholme"] ), zn( Z["Ghostlands"] ) };
    zones[zn( Z["Silithus"] )].zones = { zn( Z["Un'Goro Crater"] ), zn( Z["Ahn'Qiraj"] ), zn( Z["Temple of Ahn'Qiraj"] ) };
    zones[zn( Z["Winterspring"] )].zones = { zn( Z["Moonglade"] ), zn( Z["Felwood"] ) };

    zones[zn( Z["Isle of Quel'Danas"] )].zones = { zn( Z["Magisters' Terrace"] ), zn( Z["Sunwell Plateau"] ), zn( Z["Ironforge"] ), zn( Z["Silvermoon City"] ) };

    zones[zn( Z["Silvermoon City"] )].zones = { zn( Z["Eversong Woods"] ), zn(L["Orb of Translocation"] ), zn( Z["Isle of Quel'Danas"] ), zn( L["Battlemaster"], 36 ), zn( L["Battlemaster"], 37 ), zn( L["Battlemaster"], 38 ), zn( L["Battlemaster"], 39 ), zn( L["Battlemaster"], 40 ) };
    zones[zn( Z["Eversong Woods"] )].zones = { zn( Z["Ghostlands"] ), zn( Z["Silvermoon City"] ) };
    zones[zn( Z["Ghostlands"] )].zones = { zn( Z["Eversong Woods"] ), zn( Z["Eastern Plaguelands"] ), zn( Z["Zul'Aman"] ) };

    zones[zn( Z["Auberdine"] )].zones = { zn( Z["Darkshore"] ), zn( L["Boat"], 2 ), zn( L["Boat"], 3 ), zn( L["Boat"], 5 ) };
    zones[zn( Z["Azuremyst Isle"] )].zones = { zn( Z["Bloodmyst Isle"] ), zn( Z["The Exodar"] ), zn( L["Boat"], 5 ) };
    zones[zn( Z["Bloodmyst Isle"] )].zones = { zn( Z["Azuremyst Isle"] ) };
    zones[zn( Z["The Exodar"] )].zones = { zn( Z["Azuremyst Isle"] ), zn( L["Battlemaster"], 16 ), zn( L["Battlemaster"], 17 ), zn( L["Battlemaster"], 18 ), zn( L["Battlemaster"], 19 ), zn( L["Battlemaster"], 20 ) };

    zones[zn( Z["Shattrath City"] )].zones = { zn( Z["Terokkar Forest"] ), zn( L["Portal"], 1 ), zn( L["Portal"], 2 ), zn( L["Portal"], 3 ), zn( L["Portal"], 4 ), zn( L["Portal"], 5 ), zn( L["Portal"], 6 ), zn( L["Portal"], 7 ), zn( L["Portal"], 8 ), zn( L["Portal"], 9 ), zn( L["Battlemaster"], 41 ), zn( L["Battlemaster"], 42 ), zn( L["Battlemaster"], 43 ), zn( L["Battlemaster"], 44 ), zn( L["Battlemaster"], 45 ), zn( L["Battlemaster"], 46 ), zn( L["Battlemaster"], 47 ), zn( L["Battlemaster"], 48 ), zn( L["Battlemaster"], 49 ), zn( L["Battlemaster"], 50 ) };
    zones[zn( Z["Hellfire Peninsula"] )].zones = { zn( Z["Zangarmarsh"] ), zn( Z["Terokkar Forest"] ), zn( Z["Hellfire Citadel"] ), zn( Z["The Dark Portal"] ) };
    zones[zn( Z["Zangarmarsh"] )].zones = { zn( Z["Hellfire Peninsula"] ), zn( Z["Terokkar Forest"] ), zn( Z["Nagrand"] ), zn( Z["Blade's Edge Mountains"] ), zn( Z["Coilfang Reservoir"] ) };
    zones[zn( Z["Terokkar Forest"] )].zones = { zn( Z["Hellfire Peninsula"] ), zn( Z["Zangarmarsh"] ), zn( Z["Nagrand"] ), zn( Z["Shadowmoon Valley"] ), zn( Z["The Bone Wastes"] ), zn( Z["Shattrath City"] ) };
    zones[zn( Z["Nagrand"] )].zones = { zn( Z["Zangarmarsh"] ), zn( Z["Terokkar Forest"] ) };
    zones[zn( Z["Blade's Edge Mountains"] )].zones = { zn( Z["Zangarmarsh"] ), zn( Z["Netherstorm"] ), zn( Z["Gruul's Lair"] ) };
    zones[zn( Z["Netherstorm"] )].zones = { zn( Z["Blade's Edge Mountains"] ), zn( Z["Tempest Keep"] ) };
    zones[zn( Z["Shadowmoon Valley"] )].zones = { zn( Z["Terokkar Forest"] ), zn( Z["Black Temple"] ) };
    
    zones[zn( Z["Ragefire Chasm"] )].zones = { zn( Z["Orgrimmar"] ) };
    zones[zn( Z["Wailing Caverns"] )].zones = { zn( Z["The Barrens"] ) };
    zones[zn( Z["The Deadmines"] )].zones = { zn( Z["Westfall"] ) };
    zones[zn( Z["The Stockade"] )].zones = { zn( Z["Stormwind City"] ) };
    zones[zn( Z["Shadowfang Keep"] )].zones = { zn( Z["Silverpine Forest"] ) };
    zones[zn( Z["Blackfathom Deeps"] )].zones = { zn( Z["Ashenvale"] ) };
    zones[zn( Z["Gnomeregan"] )].zones = { zn( Z["Dun Morogh"] ) };
    zones[zn( Z["Razorfen Kraul"] )].zones = { zn( Z["The Barrens"] ) };
    zones[zn( Z["Scarlet Monastery"] )].zones = { zn( Z["Tirisfal Glades"] ) };
    zones[zn( Z["Razorfen Downs"] )].zones = { zn( Z["The Barrens"] ) };
    zones[zn( Z["Uldaman"] )].zones = { zn( Z["Badlands"] ) };
    zones[zn( Z["Maraudon"] )].zones = { zn( Z["Desolace"] ) };
    zones[zn( Z["Zul'Farrak"] )].zones = { zn( Z["Tanaris"] ) };
    zones[zn( Z["The Temple of Atal'Hakkar"] )].zones = { zn( Z["Swamp of Sorrows"] ) };
    zones[zn( Z["Blackrock Depths"] )].zones = { zn( Z["Blackrock Mountain"] ), zn( Z["Molten Core"] ) };
    zones[zn( Z["Blackrock Spire"] )].zones = { zn( Z["Blackrock Mountain"] ), zn( Z["Blackwing Lair"] ) };
    zones[zn( Z["Stratholme"] )].zones = { zn( Z["Eastern Plaguelands"] ) };
    zones[zn( Z["Dire Maul"] )].zones = { zn( Z["Feralas"] ) };
    zones[zn( Z["Scholomance"] )].zones = { zn( Z["Western Plaguelands"] ) };
    zones[zn( Z["Warsong Gulch"], 1 )].zones = { zn( Z["The Barrens"] ) };
    zones[zn( Z["Warsong Gulch"], 2 )].zones = { zn( Z["Ashenvale"] ) };
    zones[zn( Z["Arathi Basin"] )].zones = { zn( Z["Arathi Highlands"] ) };
    zones[zn( Z["Alterac Valley"] )].zones = { zn( Z["Alterac Mountains"] ) };
    zones[zn( Z["Blackwing Lair"] )].zones = { zn( Z["Blackrock Spire"] ) };
    zones[zn( Z["Molten Core"] )].zones = { zn( Z["Blackrock Depths"] ) };
    zones[zn( Z["Onyxia's Lair"] )].zones = { zn( Z["Dustwallow Marsh"] ) };
    zones[zn( Z["Zul'Gurub"] )].zones = { zn( Z["Stranglethorn Vale"] ) };
    zones[zn( Z["Ahn'Qiraj"] )].zones = { zn( Z["Silithus"] ) };
    zones[zn( Z["Temple of Ahn'Qiraj"] )].zones = { zn( Z["Silithus"] ) };
    zones[zn( Z["Zul'Aman"] )].zones = { zn( Z["Ghostlands"] ) };

    zones[zn( Z["Caverns of Time"] )].zones = { zn( Z["Tanaris"] ), zn( Z["Old Hillsbrad Foothills"] ), zn( Z["The Black Morass"] ), zn( Z["Hyjal Summit"] ), zn( Z["The Culling of Stratholme"] ) };
    zones[zn( Z["Old Hillsbrad Foothills"] )].zones = { zn( Z["Caverns of Time"] ) };
    zones[zn( Z["The Black Morass"] )].zones = { zn( Z["Caverns of Time"] ) };
    zones[zn( Z["Hyjal Summit"] )].zones = { zn( Z["Caverns of Time"] ) };
    zones[zn( Z["The Culling of Stratholme"] )].zones = { zn( Z["Caverns of Time"] ) };

    zones[zn( Z["Hellfire Citadel"] )].zones = { zn( Z["Hellfire Peninsula"] ), zn( Z["Hellfire Ramparts"] ), zn( Z["The Blood Furnace"] ), zn( Z["The Shattered Halls"] ), zn( Z["Magtheridon's Lair"] ) };
    zones[zn( Z["Hellfire Ramparts"] )].zones = { zn( Z["Hellfire Citadel"] ) };
    zones[zn( Z["The Blood Furnace"] )].zones = { zn( Z["Hellfire Citadel"] ) };
    zones[zn( Z["The Shattered Halls"] )].zones = { zn( Z["Hellfire Citadel"] ) };
    zones[zn( Z["Magtheridon's Lair"] )].zones = { zn( Z["Hellfire Citadel"] ) };

    zones[zn( Z["Tempest Keep"] )].zones = { zn( Z["Netherstorm"] ), zn( Z["The Mechanar"] ), zn( Z["The Botanica"] ), zn( Z["The Arcatraz"] ), zn( Z["The Eye"] ) };
    zones[zn( Z["The Mechanar"] )].zones = { zn( Z["Tempest Keep"] ) };
    zones[zn( Z["The Botanica"] )].zones = { zn( Z["Tempest Keep"] ) };
    zones[zn( Z["The Arcatraz"] )].zones = { zn( Z["Tempest Keep"] ) };
    zones[zn( Z["The Eye"] )].zones = { zn( Z["Tempest Keep"] ) };

    zones[zn( Z["Coilfang Reservoir"] )].zones = { zn( Z["Zangarmarsh"] ), zn( Z["The Slave Pens"] ), zn( Z["The Underbog"] ), zn( Z["The Steamvault"] ), zn( Z["Serpentshrine Cavern"] ) };
    zones[zn( Z["The Slave Pens"] )].zones = { zn( Z["Coilfang Reservoir"] ) };
    zones[zn( Z["The Underbog"] )].zones = { zn( Z["Coilfang Reservoir"] ) };
    zones[zn( Z["The Steamvault"] )].zones = { zn( Z["Coilfang Reservoir"] ) };
    zones[zn( Z["Serpentshrine Cavern"] )].zones = { zn( Z["Coilfang Reservoir"] ) };

    zones[zn( Z["The Bone Wastes"] )].zones = { zn( Z["Terokkar Forest"] ), zn( Z["Auchenai Crypts"] ), zn( Z["Shadow Labyrinth"] ), zn( Z["Sethekk Halls"] ), zn( Z["Mana-Tombs"] ) };
    zones[zn( Z["Auchenai Crypts"] )].zones = { zn( Z["The Bone Wastes"] ) };
    zones[zn( Z["Shadow Labyrinth"] )].zones = { zn( Z["The Bone Wastes"] ) };
    zones[zn( Z["Sethekk Halls"] )].zones = { zn( Z["The Bone Wastes"] ) };
    zones[zn( Z["Mana-Tombs"] )].zones = { zn( Z["The Bone Wastes"] ) };

    zones[zn( Z["Gruul's Lair"] )].zones = { zn( Z["Blade's Edge Mountains"] ) };
    zones[zn( Z["Black Temple"] )].zones = { zn( Z["Shadowmoon Valley"] ) };

--["Isle of Quel'Danas"]
    zones[zn( Z["Magisters' Terrace"] )].zones = { zn( Z["Isle of Quel'Danas"] ) };
    zones[zn( Z["Sunwell Plateau"] )].zones = { zn( Z["Isle of Quel'Danas"] ) };

    zones[zn( Z["Gadgetzan"] )].zones = { zn( Z["Tanaris"] ) };
    zones[zn( Z["Deeprun Tram"] )].zones = { zn( Z["Stormwind City"] ), zn( Z["Ironforge"] ) };
    zones[zn( Z["Theramore Isle"] )].zones = { zn( Z["Dustwallow Marsh"] ), zn( L["Boat"], 1 ) };
    zones[zn( Z["Booty Bay"] )].zones = { zn( Z["Stranglethorn Vale"] ), zn( L["Boat"], 4 ) };
    zones[zn( Z["Ratchet"] )].zones = { zn( Z["The Barrens"] ), zn( L["Boat"], 4 ) };
    zones[zn( Z["Menethil Harbor"] )].zones = { zn( Z["Wetlands"] ), zn( L["Boat"], 1 ), zn( L["Boat"], 7 ) };
    zones[zn( Z["Auberdine"] )].zones = { zn( Z["Darkshore"] ), zn( L["Boat"], 2 ), zn( L["Boat"], 3 ) };
    zones[zn( Z["Grom'gol Base Camp"] )].zones = { zn( Z["Stranglethorn Vale"] ), zn( L["Zeppelin"], 2 ), zn( L["Zeppelin"], 3 ) };
    zones[zn( Z["Blackrock Mountain"] )].zones = { zn( Z["Searing Gorge"] ), zn( Z["Burning Steppes"] ), zn( Z["Blackrock Depths"] ), zn( Z["Blackrock Spire"] ) };

    zones[zn( L["Portal"], 1 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Isle of Quel'Danas"] ) };
    zones[zn( L["Portal"], 2 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Ironforge"] ) };
    zones[zn( L["Portal"], 3 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Stormwind City"] ) };
    zones[zn( L["Portal"], 4 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Darnassus"] ) };
    zones[zn( L["Portal"], 5 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Undercity"] ) };
    zones[zn( L["Portal"], 6 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Orgrimmar"] ) };
    zones[zn( L["Portal"], 7 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Thunder Bluff"] ) };
    zones[zn( L["Portal"], 8 )].zones = { zn( Z["Shattrath City"] ), zn( Z["The Exodar"] ) };
    zones[zn( L["Portal"], 9 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Silvermoon City"] ) };
    zones[zn( L["Portal"], 10 )].zones = { zn( Z["Dalaran"] ), zn( Z["Ironforge"] ) };
    zones[zn( L["Portal"], 11 )].zones = { zn( Z["Dalaran"] ), zn( Z["Stormwind City"] ) };
    zones[zn( L["Portal"], 12 )].zones = { zn( Z["Dalaran"] ), zn( Z["Darnassus"] ) };
    zones[zn( L["Portal"], 13 )].zones = { zn( Z["Dalaran"] ), zn( Z["The Exodar"] ) };
    zones[zn( L["Portal"], 14 )].zones = { zn( Z["Dalaran"] ), zn( Z["Shattrath City"] ) };
    zones[zn( L["Portal"], 15 )].zones = { zn( Z["Dalaran"] ), zn( Z["Undercity"] ) };
    zones[zn( L["Portal"], 16 )].zones = { zn( Z["Dalaran"] ), zn( Z["Orgrimmar"] ) };
    zones[zn( L["Portal"], 17 )].zones = { zn( Z["Dalaran"] ), zn( Z["Thunder Bluff"] ) };
    zones[zn( L["Portal"], 18 )].zones = { zn( Z["Dalaran"] ), zn( Z["Silvermoon City"] ) };
    zones[zn( L["Portal"], 19 )].zones = { zn( Z["Dalaran"] ), zn( Z["Shattrath City"] ) };
    zones[zn( L["Portal"], 20 )].zones = { zn( Z["Dalaran"] ), zn( Z["Wintergrasp"] ) };
    zones[zn( L["Portal"], 21 )].zones = { zn( Z["Dalaran"] ), zn( Z["Wintergrasp"] ) };
    zones[zn( L["Portal"], 22 )].zones = { zn( Z["Dalaran"] ), zn( Z["Caverns of Time"] ) };
    zones[zn( L["Portal"], 23 )].zones = { zn( Z["Dalaran"] ), zn( Z["Caverns of Time"] ) };
    zones[zn( L["Portal"], 24 )].zones = { zn( Z["Dalaran"] ), zn( Z["Warsong Gulch"], 2 ) };
    zones[zn( L["Portal"], 25 )].zones = { zn( Z["Dalaran"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Portal"], 26 )].zones = { zn( Z["Dalaran"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Portal"], 27 )].zones = { zn( Z["Dalaran"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Portal"], 28 )].zones = { zn( Z["Dalaran"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Portal"], 29 )].zones = { zn( Z["Dalaran"] ), zn( Z["Warsong Gulch"], 1 ) };
    zones[zn( L["Portal"], 30 )].zones = { zn( Z["Dalaran"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Portal"], 31 )].zones = { zn( Z["Dalaran"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Portal"], 32 )].zones = { zn( Z["Dalaran"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Portal"], 33 )].zones = { zn( Z["Dalaran"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Portal"], 34 )].zones = { zn( Z["Dalaran"] ), zn( L["Vault of Archavon"] ) };
    zones[zn( L["Portal"], 35 )].zones = { zn( Z["Dalaran"] ), zn( L["Vault of Archavon"] ) };

    zones[zn( L["Battlemaster"], 1 )].zones = { zn( Z["Stormwind City"] ), zn( Z["Warsong Gulch"], 2 ) };
    zones[zn( L["Battlemaster"], 2 )].zones = { zn( Z["Stormwind City"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 3 )].zones = { zn( Z["Stormwind City"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 4 )].zones = { zn( Z["Stormwind City"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 5 )].zones = { zn( Z["Stormwind City"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 6 )].zones = { zn( Z["Ironforge"] ), zn( Z["Warsong Gulch"], 2 ) };
    zones[zn( L["Battlemaster"], 7 )].zones = { zn( Z["Ironforge"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 8 )].zones = { zn( Z["Ironforge"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 9 )].zones = { zn( Z["Ironforge"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 10 )].zones = { zn( Z["Ironforge"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 11 )].zones = { zn( Z["Darnassus"] ), zn( Z["Warsong Gulch"], 2 ) };
    zones[zn( L["Battlemaster"], 12 )].zones = { zn( Z["Darnassus"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 13 )].zones = { zn( Z["Darnassus"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 14 )].zones = { zn( Z["Darnassus"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 15 )].zones = { zn( Z["Darnassus"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 16 )].zones = { zn( Z["The Exodar"] ), zn( Z["Warsong Gulch"], 2 ) };
    zones[zn( L["Battlemaster"], 17 )].zones = { zn( Z["The Exodar"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 18 )].zones = { zn( Z["The Exodar"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 19 )].zones = { zn( Z["The Exodar"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 20 )].zones = { zn( Z["The Exodar"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 21 )].zones = { zn( Z["Undercity"] ), zn( Z["Warsong Gulch"], 1 ) };
    zones[zn( L["Battlemaster"], 22 )].zones = { zn( Z["Undercity"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 23 )].zones = { zn( Z["Undercity"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 24 )].zones = { zn( Z["Undercity"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 25 )].zones = { zn( Z["Undercity"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 26 )].zones = { zn( Z["Orgrimmar"] ), zn( Z["Warsong Gulch"], 1 ) };
    zones[zn( L["Battlemaster"], 27 )].zones = { zn( Z["Orgrimmar"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 28 )].zones = { zn( Z["Orgrimmar"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 29 )].zones = { zn( Z["Orgrimmar"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 30 )].zones = { zn( Z["Orgrimmar"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 31 )].zones = { zn( Z["Thunder Bluff"] ), zn( Z["Warsong Gulch"], 1 ) };
    zones[zn( L["Battlemaster"], 32 )].zones = { zn( Z["Thunder Bluff"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 33 )].zones = { zn( Z["Thunder Bluff"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 34 )].zones = { zn( Z["Thunder Bluff"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 35 )].zones = { zn( Z["Thunder Bluff"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 36 )].zones = { zn( Z["Silvermoon City"] ), zn( Z["Warsong Gulch"], 1 ) };
    zones[zn( L["Battlemaster"], 37 )].zones = { zn( Z["Silvermoon City"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 38 )].zones = { zn( Z["Silvermoon City"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 39 )].zones = { zn( Z["Silvermoon City"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 40 )].zones = { zn( Z["Silvermoon City"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 41 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Warsong Gulch"], 2 ) };
    zones[zn( L["Battlemaster"], 42 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 43 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 44 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 45 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Strand of the Ancients"] ) };
    zones[zn( L["Battlemaster"], 46 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Warsong Gulch"], 1 ) };
    zones[zn( L["Battlemaster"], 47 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Arathi Basin"] ) };
    zones[zn( L["Battlemaster"], 48 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Alterac Valley"] ) };
    zones[zn( L["Battlemaster"], 49 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Eye of the Storm"] ) };
    zones[zn( L["Battlemaster"], 50 )].zones = { zn( Z["Shattrath City"] ), zn( Z["Strand of the Ancients"] ) };

    zones[zn( L["Boat"], 1 )].zones = { zn( Z["Theramore Isle"] ), zn( Z["Menethil Harbor"] ) };
    zones[zn( L["Boat"], 2 )].zones = { zn( Z["Stormwind City"] ), zn( Z["Auberdine"] ) };
    zones[zn( L["Boat"], 3 )].zones = { zn( Z["Darnassus"] ), zn( Z["Auberdine"] ) };
    zones[zn( L["Boat"], 4 )].zones = { zn( Z["Booty Bay"] ), zn( Z["Ratchet"] ) };
    zones[zn( L["Boat"], 5 )].zones = { zn( Z["Azuremyst Isle"] ), zn( Z["Auberdine"] ) };
    zones[zn( L["Boat"], 6 )].zones = { zn( Z["Stormwind City"] ), zn( Z["Borean Tundra"] ) };
    zones[zn( L["Boat"], 7 )].zones = { zn( Z["Menethil Harbor"] ), zn( Z["Howling Fjord"] ) };
    zones[zn( L["Boat"], 8 )].zones = { zn( Z["Borean Tundra"] ), zn( Z["Dragonblight"] ) };
    zones[zn( L["Boat"], 9 )].zones = { zn( Z["Howling Fjord"] ), zn( Z["Dragonblight"] ) };
    zones[zn( L["Zeppelin"], 1 )].zones = { zn( Z["Tirisfal Glades"] ), zn( Z["Durotar"] ) };
    zones[zn( L["Zeppelin"], 2 )].zones = { zn( Z["Tirisfal Glades"] ), zn( Z["Grom'gol Base Camp"] ) };
    zones[zn( L["Zeppelin"], 3 )].zones = { zn( Z["Durotar"] ), zn( Z["Grom'gol Base Camp"] ) };
    zones[zn( L["Zeppelin"], 4 )].zones = { zn( Z["Durotar"] ), zn( Z["Borean Tundra"] ) };
    zones[zn( L["Zeppelin"], 5 )].zones = { zn( Z["Tirisfal Glades"] ), zn( Z["Howling Fjord"] ) };
    zones[zn(L["Orb of Translocation"] )].zones = { zn( Z["Silvermoon City"] ), zn( Z["Undercity"] ) };
    zones[zn( Z["The Dark Portal"] )].zones = { zn( Z["Hellfire Peninsula"] ), zn( Z["Blasted Lands"] ) };

--Northrend
    zones[zn( Z["Howling Fjord"] )].zones = { zn( Z["Grizzly Hills"] ), zn( Z["Utgarde Keep"], 1 ), zn( L["Zeppelin"], 6 ), zn( L["Boat"], 9 ) };
    zones[zn( Z["Utgarde Keep"], 1 )].zones = { zn( Z["Howling Fjord"] ), zn( Z["Utgarde Keep"], 2 ), zn( Z["Utgarde Pinnacle"] ) };
    zones[zn( Z["Utgarde Keep"], 2 )].zones = { zn( Z["Utgarde Keep"], 1 ) };
    zones[zn( Z["Utgarde Pinnacle"] )].zones = { zn( Z["Utgarde Keep"], 1 ) };

    zones[zn( Z["Borean Tundra"] )].zones = { zn( Z["Dragonblight"] ), zn( Z["Sholazar Basin"] ), zn( Z["Coldarra"] ), zn( L["Boat"], 8 ), zn( L["Zeppelin"], 6 ) };
    zones[zn( Z["Coldarra"] )].zones = { zn( Z["Borean Tundra"] ), zn( Z["The Nexus"] ), zn( Z["The Oculus"] ), zn( Z["The Eye of Eternity"] ) };
    zones[zn( Z["The Nexus"] )].zones = { zn( Z["Coldarra"] ) };
    zones[zn( Z["The Oculus"] )].zones = { zn( Z["Coldarra"] ) };
    zones[zn( Z["The Eye of Eternity"] )].zones = { zn( Z["The Nexus"], 1 ) };

    zones[zn( Z["Dragonblight"] )].zones = { zn( Z["Borean Tundra"] ), zn( Z["Zul'Drak"] ), zn( Z["Grizzly Hills"] ), zn( Z["Crystalsong Forest"] ), zn( Z["Azjol-Nerub"] ), zn( Z["Ahn'kahet: The Old Kingdom"] ), zn( Z["The Obsidian Sanctum"] ), zn( Z["Naxxramas"] ) };
    zones[zn( Z["Azjol-Nerub"] )].zones = { zn( Z["Dragonblight"] ) };
    zones[zn( Z["Ahn'kahet: The Old Kingdom"] )].zones = { zn( Z["Dragonblight"] ) };
    zones[zn( Z["The Obsidian Sanctum"] )].zones = { zn( Z["Dragonblight"] ) };
    zones[zn( Z["Naxxramas"] )].zones = { zn( Z["Dragonblight"] ) };

    zones[zn( Z["Grizzly Hills"] )].zones = { zn( Z["Howling Fjord"] ), zn( Z["Zul'Drak"] ), zn( Z["Dragonblight"] ), zn( Z["Drak'Tharon Keep"] ) };
    zones[zn( Z["Drak'Tharon Keep"] )].zones = { zn( Z["Grizzly Hills"] ) };

    zones[zn( Z["Zul'Drak"] )].zones = { zn( Z["Grizzly Hills"] ), zn( Z["Crystalsong Forest"] ), zn( Z["Dragonblight"] ), zn( Z["Gundrak"] ) };
    zones[zn( Z["Gundrak"] )].zones = { zn( Z["Zul'Drak"] ) };

    zones[zn( Z["Sholazar Basin"] )].zones = { zn( Z["Borean Tundra"] ) };

    zones[zn( Z["Crystalsong Forest"] )].zones = { zn( Z["Dalaran"] ), zn( Z["Zul'Drak"] ), zn( Z["Dragonblight"] ), zn( Z["Icecrown"] ), zn( Z["The Storm Peaks"] ) };
    zones[zn( Z["Dalaran"] )].zones = { zn( Z["Crystalsong Forest"] ), zn( Z["The Violet Hold"] ), zn( L["Portal"], 10 ), zn( L["Portal"], 11 ), zn( L["Portal"], 12 ), zn( L["Portal"], 13 ), zn( L["Portal"], 14 ), zn( L["Portal"], 15 ), zn( L["Portal"], 16 ), zn( L["Portal"], 17 ), zn( L["Portal"], 18 ), zn( L["Portal"], 19 ), zn( L["Portal"], 20 ), zn( L["Portal"], 21 ), zn( L["Portal"], 22 ), zn( L["Portal"], 23 ), zn( L["Portal"], 24 ), zn( L["Portal"], 25 ), zn( L["Portal"], 26 ), zn( L["Portal"], 27 ), zn( L["Portal"], 28 ), zn( L["Portal"], 29 ), zn( L["Portal"], 30 ), zn( L["Portal"], 31 ), zn( L["Portal"], 32 ), zn( L["Portal"], 33 ), zn( L["Portal"], 34 ), zn( L["Portal"], 35 ) };
    zones[zn( Z["The Violet Hold"] )].zones = { zn( Z["Dalaran"] ) };

    zones[zn( Z["The Storm Peaks"] )].zones = { zn( Z["Crystalsong Forest"] ), zn( Z["Ulduar"] ) };
    zones[zn( Z["Ulduar"] )].zones = { zn( Z["The Storm Peaks"] ), zn( Z["Halls of Stone"] ), zn( Z["Halls of Lightning"] ) };
--    zones[zn( Z["Ulduar"] )].zones = { zn( Z["The Storm Peaks"] ), zn( Z["Halls of Stone"] ), zn( Z["Halls of Lightning"] ), zn( Z["Uldar Raid"] ) };
    zones[zn( Z["Halls of Stone"] )].zones = { zn( Z["Ulduar"] ) };
    zones[zn( Z["Halls of Lightning"] )].zones = { zn( Z["Ulduar"] ) };
--    zones[zn( Z["Ulduar Raid"] )].zones = { zn( Z["Ulduar"] ) };

    zones[zn( Z["Icecrown"] )].zones = { zn( Z["Crystalsong Forest"] ), zn( Z["Icecrown Citadel"] ) };
    zones[zn( Z["Icecrown Citadel"] )].zones = { zn( Z["Icecrown"] ) };

    zones[zn( Z["Wintergrasp"] )].zones = { zn( L["Vault of Archavon"] ) };
    zones[zn( L["Vault of Archavon"] )].zones = { zn( Z["Wintergrasp"] ) };
end--do

-- Font Colors
local HEX_WHITE = "|cffffffff";
local HEX_RED = "|cffff0000";
local HEX_ORANGE = "|cffff7700";
local HEX_YELLOW = "|cffffff00";
local HEX_GREEN = "|cff00ff00";
local HEX_GRAY = "|cff777777";
local HEX_LIGHTBLUE = "|cff77ccff";

function Broker_RecZone:GetZoneColor( zone )
    -- PlayerLevel <= low for zone - 4  == RED
    -- PlayerLevel <  low for zone      == ORANGE
    -- PlayerLevel <= high              == YELLOW
    -- PlayerLevel <  high + 5          == GREEN
    -- Otherwise                        == GRAY
    local type = zone.type;
    local low = zone.low;
    local high = zone.high;
    local vars = self.vars;
    local playerLevel = vars.playerLevel;

    if( ( type == RZ_CITY ) and ( zone.faction == vars.playerFaction ) ) then
        return HEX_WHITE;
    elseif( type == RZ_SANCTUARY ) then
        return HEX_LIGHTBLUE;
    elseif( ( type == RZ_PVP ) or ( playerLevel <= ( low - 4 ) ) ) then
        return HEX_RED;
    elseif( playerLevel < low ) then
        return HEX_ORANGE;
    elseif( playerLevel <= high ) then
        return HEX_YELLOW;
    elseif( playerLevel < ( high + 5 ) ) then
        return HEX_GREEN;
    else
        return HEX_GRAY;
    end--if
end--Broker_RecZone:GetZoneColor( zone )

function Broker_RecZone:GetInstanceText( instance, colors )
    local color = HEX_WHITE;
    local text = "";

    if( instance == nil ) then
        return "";
    end--if
    if( colors ) then
        color = self:GetZoneColor( instance );
    end--if
    if( instance.type == RZ_RAID ) then
        if( type( instance.size ) == "number" ) then
            text = instance.name .. L[" (Raid %i)"]:format( instance.size );
        else
            text = instance.name .. L[" (Raid %i or %i)"]:format( instance.size[1], instance.size[2] );
        end--if
    elseif( instance.type ~= RZ_INSTANCE ) then
        text = instance.name .. self.vars.types[instance.type];
    elseif( instance.low == instance.high ) then
        text = instance.name .. self.vars.types[instance.type] .. (" (%i+)"):format( instance.low );
    else
        text = instance.name .. L[" ( %i+ to %i+)"]:format( instance.low, instance.high );
    end--if
    return color .. text .. "|r";
end--Broker_RecZone:GetInstanceText( instance, colors )

function Broker_RecZone:GetZoneText( zone, colors )
    local color = HEX_WHITE;

    if( zone == nil ) then
        return "";
    end
    if( colors ) then
        color = self:GetZoneColor( zone );
    end
    
    local faction = zone.faction;
    local type = zone.type;

    -- No level range for friendly city zones
    if( ( ( type == RZ_CITY ) or ( type == RZ_TRANSPORT ) ) and ( faction == self.vars.playerFaction ) ) then
        text = zone.name;
        color = HEX_WHITE;
    elseif( faction == RZ_SANCTUARY ) then
        text = zone.name;
        color = HEX_LIGHTBLUE;
    elseif( ( type == RZ_INSTANCE ) or ( type == RZ_COLLECTION ) ) then
        if( zone.low == zone.high ) then
            text = zone.name .. (" (%i+)"):format( zone.low ); --  (60+)
        else
            text = zone.name .. L[" ( %i+ to %i+)"]:format( zone.low, zone.high ); --  (50+ - 58+)
        end--if
    elseif( ( type == RZ_BATTLEGROUND  ) or ( type == RZ_PVP ) or ( type == RZ_ARENA ) ) then
        text = zone.name .. self.vars.types[instance.type];
    elseif( type == RZ_RAID ) then
        if( type( zone.size ) == "number" ) then
            text = zone.name .. L[" (Raid %i)"]:format( zone.size );
        else
            text = zone.name .. L[" (Raid %i or %i)"]:format( zone.size[1], zone.size[2] );
        end--if
    else
        text = zone.name .. L[" ( %i to %i)"]:format( zone.low, zone.high ); -- (1 - 12 )
    end--if
    -- Add zone name and level range.
    return color .. text .. "|r";
end--Broker_RecZone:GetZoneText( zone, colors )

function Broker_RecZone:GetFactionColor( zone, colors )
    if( not zone ) then
        return "";
    end--if
    if( colors ) then
        local faction = zone.faction;

        if( faction == RZ_CONTESTED ) then
            return HEX_YELLOW;
        elseif( faction == RZ_SANCTUARY ) then
            return HEX_LIGHTBLUE;
        elseif( faction == self.vars.playerFaction ) then
            return HEX_GREEN;
        else
            return HEX_RED;
        end--if
    end--if
    return "";
end--Broker_RecZone:GetFactionColor( zone, colors )

function Broker_RecZone:GetFactionText( zone, colors )
    local color = HEX_WHITE;

    if( zone == nil ) then
        return "";
    end
    if( colors ) then
        color = self:GetFactionColor( zone, colors );
    end
    return color .. self.vars.factions[zone.faction] .. "|r";
end--Broker_RecZone:GetFactionText( zone, colors )

function Broker_RecZone:GetContinentText( zone, colors )
    local color = HEX_WHITE;

    if( zone == nil ) then
        return "";
    end--if
    if( colors ) then
        if( zone.continent == self.vars.current_continent ) then
            color = HEX_GREEN;
        else
            color = HEX_YELLOW;
        end--if
    end--if
    return color .. self.vars.continents[zone.continent] .. "|r";
end--Broker_RecZone:GetContinentText( zone, colors )

function Broker_RecZone:GetLevelText( zone, colors )
    local color = HEX_WHITE;

    if ( colors ) then
        color = self:GetZoneColor( zone );
    end
    if( ( zone.type == RZ_CITY ) and ( ( zone.faction == self.vars.playerFaction ) or ( zone.faction == RZ_SANCTUARY ) ) ) then
        return HEX_WHITE .. self.vars.types[zone.type] .. "|r";
    elseif (zone.faction == RZ_SANCTUARY) then
        return HEX_LIGHTBLUE .. self.vars.types[zone.type] .. "|r";
    elseif( ( zone.type == RZ_INSTANCE ) or ( zone.type == RZ_COLLECTION ) ) then
        if( zone.low == zone.high ) then
            return color .. ("%i+|r"):format( zone.low ); -- 60+
        else
            return color .. ("%i+-%i+|r"):format( zone.low , zone.high ); --  50+-58+
        end--if
    elseif( ( zone.type == RZ_BATTLEGROUND ) or ( zone.type == RZ_PVP ) or ( zone.type == RZ_ARENA ) ) then
        return color .. self.vars.types[zone.type] .. "|r";
    elseif( zone.type == RZ_RAID ) then
        if( type( zone.size ) == "number" ) then
            text = color .. L[" (Raid %i)"]:format( zone.size ) .. "|r";
        else
            text = color .. L[" (Raid %i or %i)"]:format( zone.size[1], zone.size[2] ) .. "|r";
        end--if
    end--if
    if( zone.low == zone.high ) then
        return color .. tostring( zone.low ) .. "|r";
    end--if
    return color .. ("%i-%i|r"):format( zone.low , zone.high );
end--Broker_RecZone:GetLevelText( zone, colors )

function Broker_RecZone:GetPathText( zone, colors )
    local zonecolor = HEX_WHITE;
    local levelcolor = HEX_WHITE;

    if ( colors ) then
        zonecolor = self:GetFactionColor( zone, colors );
        levelcolor = self:GetZoneColor( zone );
    end
    if( ( zone.type == RZ_CITY ) or ( zone.type == RZ_TRANSPORT ) ) then
        return zonecolor .. zone.name .. "|r";
    elseif( ( zone.type == RZ_INSTANCE ) or ( zone.type == RZ_COLLECTION ) ) then
        if( zone.low == zone.high ) then
            return zonecolor .. zone.name .. "|r " .. levelcolor .. ("(%i+)|r"):format( zone.low );
        else
            return zonecolor .. zone.name .. "|r " .. levelcolor .. ("(%i+-%i+)|r"):format( zone.low, zone.high );
        end
    elseif( ( zone.type == RZ_BATTLEGROUND ) or ( zone.type == RZ_PVP ) or ( zone.type == RZ_ARENA ) ) then
        return zonecolor .. zone.name .. "|r" .. levelcolor .. self.vars.types[zone.type] .. "|r";
    elseif( zone.type == RZ_RAID ) then
        if( type( zone.size ) == "number" ) then
            text = zonecolor .. zone.name .. "|r" .. levelcolor .. L[" (Raid %i)"]:format( zone.size ) .. "|r";
        else
            text = zonecolor .. zone.name .. "|r" .. levelcolor .. L[" (Raid %i or %i)"]:format( zone.size[1], zone.size[2] ) .. "|r";
        end--if
    end
    if( zone.low == zone.high ) then
        return zonecolor .. zone.name .. "|r " .. levelcolor .. ("(%i)|r"):format( zone.low );
    end
    return zonecolor .. zone.name .. "|r " .. levelcolor .. ("(%i-%i)|r"):format( zone.low, zone.high );
end--Broker_RecZone:GetPathText( zone, colors )

function Broker_RecZone:StartCoordsUpdate()
    local x, y = GetPlayerMapPosition( "player" );

    if( x == 0 and y == 0 ) then -- instance
        self.timerHandle = self:ScheduleRepeatingTimer( self.OnCoordUpdate, 60, self );
        self.vars.playerCoordX = 0;
        self.vars.playerCoordY = 0;
        self.vars.coordtext = "";
    else
        self.timerHandle = self:ScheduleRepeatingTimer( self.OnCoordUpdate, 0.1, self );
    end--if
end--Broker_RecZone:StartCoordsUpdate()

function Broker_RecZone:StopCoordsUpdate()
    if( self.timerHandle ) then
        self:CancelRepeatingEvent( self.timerHandle );
        self.timerHandle = nil;
    end--if
    self.vars.playerCoordX = 0;
    self.vars.playerCoordY = 0;
    self.vars.coordtext = "";
end--Broker_RecZone:StopCoordsUpdate()

Broker_RecZone.vars.playerinworld = false;

function Broker_RecZone:OnInitialize()
    local AceConfigReg = LibStub( "AceConfigRegistry-3.0" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local defaults = {
        profile = {
            showCurInstance = true,
            showInstance = true,
            showBattleground = true,
            showRaid = false,
            showFaction = false,
            showContinent = false,
            showLoc = true,
            showMap = true,
            lower = 5,
            higher = 5,
            showColoredText = true,
            showZoneName = true,
            showSubZoneName = false,
            showCoords = true,
            showMapCoords = true,
            minimapBar = false,
        },
    };
    local options = {
        type = "group",
        handler = self,
        args = {
            label = {
                type = "group",
                name = L["Label"],
                desc = L["Label"],
                order = 1,
                args = {
                    zone = {
                        type = "toggle",
                        name = L["Show Zone Name"],
                        desc = L["Show Zone Name"],
                        get = function( info )
                                  return info.handler.db.profile.showZoneName;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showZoneName = value;
                                  info.handler:Update();
                              end,
                        order = 1,
                    },
                    subzone = {
                        type = "toggle",
                        name = L["Show Sub Zone Name"],
                        desc = L["Show Sub Zone Name"],
                        get = function( info )
                                  return info.handler.db.profile.showSubZoneName;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showSubZoneName = value;
                                  info.handler:Update();
                              end,
                        order = 2,
                    },
                    coords = {
                        type = "toggle",
                        name = L["Show Coords"],
                        desc = L["Show Coords"],
                        get = function( info )
                                  return info.handler.db.profile.showCoords;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showCoords = value;
                                  if( value ) then
                                      info.handler:StartCoordsUpdate();
                                  else
                                      info.handler:StopCoordsUpdate();
                                  end--if
                                  info.handler:Update();
                              end,
                        order = 3,
                    },
                },
            },
            tooltip = {
                type = "group",
                name = L["Tooltip"],
                desc = L["Tooltip"],
                order = 2,
                args = {
                    curinstance = {
                        type = "toggle",
                        name = L["Show Instances for current zone"],
                        desc = L["Show Instances for current zone"],
                        get = function( info )
                                  return info.handler.db.profile.showCurInstance;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showCurInstance = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 1,
                    },
                    faction = {
                        type = "toggle",
                        name = L["Show Faction of zones"],
                        desc = L["Show Faction of zones"],
                        get = function( info )
                                  return info.handler.db.profile.showFaction;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showFaction = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 2,
                    },
                    continent = {
                        type = "toggle",
                        name = L["Show Continent for zones"],
                        desc = L["Show Continent for zones"],
                        get = function( info )
                                  return info.handler.db.profile.showContinent;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showContinent = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 3,
                    },
                    instance = {
                        type = "toggle",
                        name = L["Show Recommended Instances"],
                        desc = L["Show Recommended Instances"],
                        get = function( info )
                                  return info.handler.db.profile.showInstance;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showInstance = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 4,
                    },
                    battleground = {
                        type = "toggle",
                        name = L["Show Battlegrounds"],
                        desc = L["Show Battlegrounds"],
                        get = function( info )
                                  return info.handler.db.profile.showBattleground;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showBattleground = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 5,
                    },
                    raid = {
                        type = "toggle",
                        name = L["Show Raid Instances"],
                        desc = L["Show Raid Instances"],
                        get = function( info )
                                  return info.handler.db.profile.showRaid;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showRaid = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 6,
                    },
                    loc = {
                        type = "toggle",
                        name = L["Show Location of Instances"],
                        desc = L["Show Location of Instances"],
                        get = function( info )
                                  return info.handler.db.profile.showLoc;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showLoc = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 7,
                    },
                    lower = {
                        type = "range",
                        name = L["Recommend lower level zones"],
                        desc = L["Recommend lower level zones"],
                        max = 10;
                        min = 0;
                        step = 1;
                        get = function( info )
                                  return info.handler.db.profile.lower;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.lower = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 8,
                    },
                    higher = {
                        type = "range",
                        name = L["Recommend higher level zones"],
                        desc = L["Recommend higher level zones"],
                        max = 10;
                        min = 0;
                        step = 1;
                        get = function( info )
                                  return info.handler.db.profile.higher;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.higher = value;
                                  info.handler:UpdateTooltip();
                              end,
                        order = 9,
                    },
                },
            },
            map = {
                type = "group",
                name = L["Map"],
                desc = L["Map"],
                order = 3,
                args = {
                    mapoverlay = {
                        type = "toggle",
                        name = L["Show Zone Levels"],
                        desc = L["Show Zone Levels"],
                        get = function( info )
                                  return info.handler.db.profile.showMap;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showMap = value;
                                  if( value ) then
                                      Broker_RecZoneMapText:Show();
                                  else
                                      Broker_RecZoneMapText:Hide();
                                  end
                              end,
                        order = 1,
                    },
                    mapcoords = {
                        type = "toggle",
                        name = L["Show Coords"],
                        desc = L["Show Coords"],
                        get = function( info )
                                  return info.handler.db.profile.showMapCoords;
                              end,
                        set = function( info, value )
                                  info.handler.db.profile.showMapCoords = value;
                                  if( value ) then
                                      Broker_RecZoneMapFrameCursorCoords:Show();
                                      Broker_RecZoneMapFramePlayerCoords:Show();
                                  else
                                      Broker_RecZoneMapFrameCursorCoords:Hide();
                                      Broker_RecZoneMapFramePlayerCoords:Hide();
                                  end--if
                              end,
                        order = 2,
                    },
                },
            },
            minimapBar = {
                type = "toggle",
                name = L["Show minimap bar"],
                desc = L["Show the bar above the minimap that tells the location and allows you to close minimap"],
                get = function( info )
                          return info.handler.db.profile.minimapBar
                      end,
                set = function( info, value )
                          info.handler.db.profile.minimapBar = value;
                          if( value ) then
                              MinimapBorderTop:Show();
                              MinimapToggleButton:Show();
                              MinimapZoneTextButton:Show();
                          else
                              MinimapBorderTop:Hide();
                              MinimapToggleButton:Hide();
                              MinimapZoneTextButton:Hide();
                          end--if
                          Jostle:Refresh();
                      end,
            },
            color = {
                type = "toggle",
                name = L["Show Colored Text"],
                desc = L["Show Colored Text"],
                get = function( info )
                          return info.handler.db.profile.showColoredText;
                      end,
                set = function( info, value )
                          info.handler.db.profile.showColoredText = value;
                          info.handler:Update();
                      end,
                order = 91,
            },
        }
    };
    
    if( RecZone ) then
        options.args.copysettings = {
            type = "execute",
            order = -1,
            name = L["Copy setting from FuBar_RecZone"],
            desc = L["Copy setting from FuBar_RecZone"],
            func = function( info )
                       info.handler.db.profile = RecZone.db.profile;
                   end,
        };
    end--if

    self.db = LibStub( "AceDB-3.0" ):New( "RecZoneDB", defaults );
    
    local profileOptions = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db );
    self.db.RegisterCallback( self, "OnProfileChanged" );
    self.db.RegisterCallback( self, "OnProfileReset" );

    AceConfigReg:RegisterOptionsTable( "Broker_RecZone Main Options", options );
    AceConfigReg:RegisterOptionsTable( "Broker_RecZone Profile", profileOptions );
    AceConfigDialog:AddToBlizOptions( "Broker_RecZone Main Options", L["Recommended Zone"] );
    AceConfigDialog:AddToBlizOptions( "Broker_RecZone Profile", L["Profile"], L["Recommended Zone"] );

    self.ldbobj = LibStub( "LibDataBroker-1.1" ):NewDataObject( L["Recommended Zone"], {
            type = "data source",
            icon = "Interface\\AddOns\\Broker_RecZone\\Icon",
            label = L["Recommended Zone"],
            text = L["Recommended Zone"], -- To get Button Bin to work correctly when autohiding text
            OnClick = function( _, button )
                          self:OnClick( button );
                      end,
            OnEnter = function( frame )
                          local tooltip = qtip:Acquire( "Broker_RecZone_Tooltip", 3, "LEFT", "CENTER", "RIGHT" );

                          self.vars.qtip = tooltip;
                          tooltip:EnableMouse( true );
                          self:UpdateTooltip();
                          tooltip:SmartAnchorTo( frame );
                          tooltip:Show();
                      end,
            OnLeave = function( frame )
                          local function HideTooltip( test )
                              if( MouseIsOver( self.vars.qtip ) ) then
                                  return;
                              end;
                              self.vars.qtip:SetScript( "OnLeave", nil );
                              qtip:Release( self.vars.qtip );
                              self.vars.qtip = nil;
                          end--HideTooltip()

                          if( MouseIsOver( self.vars.qtip ) ) then
                              self.vars.qtip:SetScript( "OnLeave", HideTooltip );
                          else
                              HideTooltip();
                          end--if
                      end,
        } );

    self.vars.continents = { GetMapContinents(), [RZ_OTHER] = "Other" };

    local mapFrame = CreateFrame( "Frame", "Broker_RecZoneMapFrame", WorldMapFrame );

    mapFrame:SetScript( "OnUpdate", self.MapFrame_OnUpdate );
    
    local text = mapFrame:CreateFontString( "Broker_RecZoneMapText", "OVERLAY" );
    local font, size = GameFontNormal:GetFont();

    text:SetFont(font, size, "OUTLINE")
    text:SetTextColor( GameFontNormal:GetTextColor() );
    text:SetShadowColor( GameFontNormal:GetShadowColor() );
    text:SetShadowOffset( GameFontNormal:GetShadowOffset() );
    text:SetPoint( "TOP", WorldMapFrameAreaLabel, "BOTTOM", 0, -5 );

    local cursorCoords = mapFrame:CreateFontString( "Broker_RecZoneMapFrameCursorCoords", "ARTWORK" );

    cursorCoords:SetFont( GameFontNormal:GetFont() );
    cursorCoords:SetTextColor( GameFontNormal:GetTextColor() );
    cursorCoords:SetShadowColor( GameFontNormal:GetShadowColor() );
    cursorCoords:SetShadowOffset( GameFontNormal:GetShadowOffset() );
    cursorCoords:SetPoint( "RIGHT", WorldMapFrame, "CENTER", -80, -367 );
    
    local playerCoords = mapFrame:CreateFontString( "Broker_RecZoneMapFramePlayerCoords", "ARTWORK" );

    playerCoords:SetFont( GameFontNormal:GetFont() );
    playerCoords:SetTextColor( GameFontNormal:GetTextColor() );
    playerCoords:SetShadowColor( GameFontNormal:GetShadowColor() );
    playerCoords:SetShadowOffset( GameFontNormal:GetShadowOffset() );
    playerCoords:SetPoint( "RIGHT", WorldMapFrame, "CENTER", 80, -367 );
end--Broker_RecZone:OnInitialize()

function Broker_RecZone:OnEnable()
    self:RegisterEvent( "ZONE_CHANGED", "Update" );
    self:RegisterEvent( "ZONE_CHANGED_INDOORS", "Update" );
    self:RegisterEvent( "ZONE_CHANGED_NEW_AREA", "Update" );
    self:RegisterEvent( "MINIMAP_ZONE_CHANGED", "Update" );
    self:RegisterEvent( "PLAYER_ENTERING_WORLD", "OnEnteringWorld" );
    self:RegisterEvent( "PLAYER_LEVEL_UP", "Update" );
    if( Broker_RecZoneMapFrame ) then
        Broker_RecZoneMapFrame:Show();
    end--if
    self.vars.playerinworld = true; -- Maybe not completly true, but good enough
    self:OnProfileChanged();
end--Broker_RecZone:OnEnable()

function Broker_RecZone:OnDisable()
    Broker_RecZoneMapFrame:Hide();
    Broker_RecZoneMapFrameCursorCoords:Hide();
    Broker_RecZoneMapFramePlayerCoords:Hide();
    if( not self.db.profile.minimapBar ) then
        MinimapBorderTop:Hide();
        MinimapToggleButton:Hide();
        MinimapZoneTextButton:Hide();
        self.db.profile.minimapBar = not self.db.profile.minimapBar;
    end--if
end--Broker_RecZone:OnDisable()

function Broker_RecZone:OnProfileReset()
    self:OnProfileChanged();
end--Broker_RecZone:OnProfileReset()

function Broker_RecZone:OnProfileChanged()
    if( self.vars.playerinworld ) then
        if( self.db.profile.showCoords ) then
            self:StartCoordsUpdate();
        else
            self:StopCoordsUpdate();
        end--if
        if( self.db.profile.showMap ) then
            Broker_RecZoneMapText:Show();
        else
            Broker_RecZoneMapText:Hide();
        end
        if( self.db.profile.showMapCoords ) then
            Broker_RecZoneMapFrameCursorCoords:Show();
            Broker_RecZoneMapFramePlayerCoords:Show();
        else
            Broker_RecZoneMapFrameCursorCoords:Hide();
            Broker_RecZoneMapFramePlayerCoords:Hide();
        end--if
        if( self.db.profile.minimapBar ) then
            MinimapBorderTop:Show();
            MinimapToggleButton:Show();
            MinimapZoneTextButton:Show();
        else
            MinimapBorderTop:Hide();
            MinimapToggleButton:Hide();
            MinimapZoneTextButton:Hide();
        end--if
        Jostle:Refresh();
        self:Update();
    end
end--Broker_RecZone:OnProfileChanged()

function Broker_RecZone:OnEnteringWorld()
    if( not self.vars.playerinworld ) then
        self.vars.playerinworld = true;
    end
    if( self.db.profile.showCoords ) then
        self:StartCoordsUpdate();
    end--if
    self:Update();
end--Broker_RecZone:OnEnteringWorld()

function Broker_RecZone:OnCoordUpdate()
    local x, y = GetPlayerMapPosition( "player" );
    local vars = self.vars;

    if( self.db.profile.showCoords and ( x ~= 0 ) and ( y ~= 0 ) ) then
        if( ( vars.playerCoordX ~= x ) or ( vars.playerCoordY ~= y ) ) then
            vars.coordtext = ("(%.0f, %.0f) "):format( x * 100, y * 100 );
            vars.playerCoordX = x;
            vars.playerCoordY = y;
            self:OnTextUpdate();
        end--if
    else
        vars.playerCoordX = x;
        vars.playerCoordY = y;
        vars.coordtext = "";
        self:OnTextUpdate();
    end--if
end--Broker_RecZone:OnCoordUpdate()

function Broker_RecZone:OnDataUpdate()
    local i;
    local vars = self.vars;
    local profile = self.db.profile;

    vars.playerLevel = UnitLevel( "player" );
    vars.zoneName = GetRealZoneText();
    vars.subZoneName = GetSubZoneText();
    if( select( 2, UnitFactionGroup( "player" ) ) == FACTION_ALLIANCE ) then
        vars.playerFaction = RZ_ALLIANCE;
    else
        vars.playerFaction = RZ_HORDE;
    end--if

    vars.zonetext = "";
    if( profile.showZoneName ) then
        vars.zonetext = self.vars.zoneName .. " ";
        if( profile.showSubZoneName ) then
            if( vars.subZoneName ~= "" ) then
                vars.zonetext = vars.zoneName .. ": " .. vars.subZoneName .. " ";
            end
        end
    elseif( profile.showSubZoneName ) then
        if( vars.subZoneName ~= "" ) then
            vars.zonetext = vars.subZoneName .. " ";
        else
            vars.zonetext = vars.zoneName .. " ";
        end--if
    end--if

    vars.leveltext = "";

    local found = false;

    for _, zone in ipairs( vars.zones ) do
        if( string.find( vars.zoneName, zone.name, 1, true ) ) then
            vars.current_continent = zone.continent;
            vars.current_zone = zone;
            vars.leveltext = self:GetLevelText( zone, profile.showColoredText );
            found = true;
            break;
        end--if
    end--for

    if( not found ) then
        if( vars.zoneName and ( vars.zoneName ~= "" ) ) then
            self:Print( "Unknown zone detected \"%s\"", vars.zoneName )
        end--if
        vars.current_continent = vars.zones[1].continent;
        vars.current_zone = vars.zones[1];
        vars.leveltext = self:GetLevelText( self.vars.zones[1], profile.showColoredText );
    end
    
    self.vars.zonetext = self:GetFactionColor( vars.current_zone, profile.showColoredText ) .. vars.zonetext;
end--Broker_RecZone:OnDataUpdate()

function Broker_RecZone:Update()
    self:OnDataUpdate();
    self:OnTextUpdate();
    self:UpdateTooltip();
end--Broker_RecZone:Update()

function Broker_RecZone:OnTextUpdate()
    local vars = self.vars;

    self.ldbobj.text = vars.zonetext .. vars.coordtext .. vars.leveltext;
end--Broker_RecZone:OnTextUpdate()

function Broker_RecZone:UpdateTooltip()
    if( self.vars.qtip ) then
        self:OnTooltipUpdate();
    end--if
end--Broker_RecZone:UpdateTooltip()

function Broker_RecZone:OnTooltipUpdate()
    local c, i;
    local text = "";
    local profile = self.db.profile;
    local lower = profile.lower;
    local higher = profile.higher;
    local colorTooltip = profile.showColoredText;
    local vars = self.vars;
    local tooltip = vars.qtip;
    local line = 0;
    local current_zone = vars.current_zone;
    local zones = vars.zones;
    local playerFaction = vars.playerFaction;

    tooltip:Clear();
    line = tooltip:AddLine();
    tooltip:SetCell( line, 1, L["Recommended Zone"], tooltip:GetHeaderFont(), tooltip:GetColumnCount() );
    line = tooltip:AddLine();
    tooltip:SetCell( line, 1, L["Current Zone:"] );
    tooltip:SetCell( line, 2, self:GetInstanceText( current_zone, false ), 2 );
    if( vars.subZoneName ~= "" ) then
        line = tooltip:AddLine();    
        tooltip:SetCell( line, 1, L["Current Sub Zone:"] );
        tooltip:SetCell( line, 2, vars.subZoneName, 2 );
    end--if    
    if( current_zone.type < RZ_INSTANCE ) then
        -- Show Coords
        if( profile.showCoords ) then
            line = tooltip:AddLine();
            tooltip:SetCell( line, 1, L["Coords:"] );
            tooltip:SetCell( line, 2, ("%.0f, %.0f"):format( vars.playerCoordX * 100, vars.playerCoordY * 100 ), 2 );
        end--if
        -- Show faction
        if( profile.showFaction ) then
            line = tooltip:AddLine();
            tooltip:SetCell( line, 1, L["Faction:"] );
            tooltip:SetCell( line, 2, self:GetFactionText( current_zone, colorTooltip ), 2 );
        end--if
        -- show continent
        if( profile.showContinent ) then
            line = tooltip:AddLine();
            tooltip:SetCell( line, 1, L["Continent:"] );
            tooltip:SetCell( line, 2, self:GetContinentText( current_zone, colorTooltip ), 2 );
        end--if
    end--if

    -- Show border zones for current zone.
    if( profile.showCurInstance ) then
        if ( current_zone.zones ) then
            for _, i in ipairs( current_zone.zones ) do
                local f = zones[i].faction;
                local t = zones[i].type;

                if( ( ( t == RZ_ZONE ) or ( t == RZ_CITY ) ) and ( ( f == RZ_CONTESTED ) or ( f == playerFaction ) or ( f == RZ_SANCTUARY ) ) ) then
                    line = tooltip:AddLine();
                    tooltip:SetCell( line, 1, L["Border Zone:"] );
                    tooltip:SetCell( line, 2, self:GetZoneText( zones[i], colorTooltip ), 2 );
                end--if
            end--for
        end--if
    end --if

    -- Show instances for current zone.
    if( profile.showCurInstance ) then
        if( current_zone.zones ) then
            for _, i in ipairs( current_zone.zones ) do
                local f = zones[i].faction;
                local t = zones[i].type;
                
                if( ( t >= RZ_INSTANCE ) and ( ( f == RZ_CONTESTED ) or ( f == playerFaction ) ) ) then
                    line = tooltip:AddLine();
                    tooltip:SetCell( line, 1, L["Instance:"] );
                    tooltip:SetCell( line, 2, self:GetInstanceText( zones[i], colorTooltip ), "RIGHT", 2 );
                end--if
            end--for
        end--if
    end--for
    
    -- Show recommended zones.
    line = tooltip:AddHeader( L["Recommended Zones:"] );

    for _,zone in ipairs( zones ) do
        if( ( zone.type == RZ_ZONE ) and
                ( ( zone.low - higher ) <= vars.playerLevel ) and
                ( ( zone.high + lower ) >= vars.playerLevel ) and
                ( ( zone.faction == RZ_CONTESTED ) or ( zone.faction == playerFaction ) ) ) then
            -- Show faction
            local factionText;

            if( profile.showFaction ) then
                factionText = self:GetFactionText( zone, colorTooltip );
            else
                factionText = "";
            end--if

            -- show continent
            local continentText;

            if( profile.showContinent ) then
                continentText = self:GetContinentText( zone, colorTooltip );
            else
                continentText = "";
            end--if

            line = tooltip:AddLine();
            tooltip:SetCell( line, 1, self:GetZoneText( zone, colorTooltip ), clickProvider, self, zone );
            tooltip:SetCell( line, 2, factionText, clickProvider, self, zone );
            tooltip:SetCell( line, 3, continentText, clickProvider, self, zone );
        end--if
    end--for

    -- Show recommended instances/battlegrounds/raids
    if( profile.showInstance or profile.showRaid or profile.showBattleground ) then
        line = tooltip:AddHeader( L["Recommended Instances:"] );
        for _, instance in ipairs( zones ) do
            local type = instance.type;

            if( ( instance.low - higher <= self.vars.playerLevel ) and
                    ( instance.high + lower >= self.vars.playerLevel ) and
                    ( ( instance.faction == RZ_CONTESTED ) or ( instance.faction == vars.playerFaction ) ) and
                    ( ( ( ( type == RZ_INSTANCE ) or ( type == RZ_COLLECTION ) ) and profile.showInstance ) or
                    ( ( ( type == RZ_RAID ) or ( type == RZ_COLLECTION ) ) and profile.showRaid ) or
                    ( ( type == RZ_BATTLEGROUND ) and profile.showBattleground ) ) ) then
                local text =  self:GetInstanceText( instance, colorTooltip );
                local locText = "";

                if( profile.showLoc and instance.zones ) then
                    locText = "<" .. zones[instance.zones[1]].name .. ">";
                end--if
                line = tooltip:AddLine();
                tooltip:SetCell( line, 1, text, clickProvider, self, instance );
                tooltip:SetCell( line, 2, locText, "RIGHT", 2, clickProvider, self, instance );
            end--if
        end--for
    end--if
    line = tooltip:AddLine();
    tooltip:SetCell( line, 1, L["Hint"], tooltip:GetColumnCount() );
end--Broker_RecZone:OnTooltipUpdate()

function Broker_RecZone:OnClick()
    if( IsShiftKeyDown() ) then
        if( ChatFrameEditBox:IsVisible() ) then
            local text = "";
            local vars = self.vars;
            local profile = self.db.profile;

            if( profile.showZoneName ) then
                text = vars.zoneName .. " ";
                if( profile.showSubZoneName ) then
                    if ( vars.subZoneName ~= "" ) then
                        text = vars.zoneName .. ": " .. vars.subZoneName .. " ";
                    end--if
                end--if
            elseif( profile.showSubZoneName ) then
                if( vars.subZoneName ~= "" ) then
                    text = vars.subZoneName .. " ";
                else
                    text = vars.zoneName .. " ";
                end--if
            end--if

            local x, y = GetPlayerMapPosition( "player" );

            text = text .. ("%.0f, %.0f"):format( x * 100, y * 100 );
            ChatFrameEditBox:Insert( text );
        end--if
    else
        InterfaceOptionsFrame_OpenToCategory( L["Recommended Zone"] );
    end--if
end--Broker_RecZone:OnTooltipUpdate()

function Broker_RecZone:OnClickZone( zone )
    local start = self:GetZoneNr( self.vars.current_zone );
    local goal = self:GetZoneNr( zone )
    local text = "";
    local tooltip =  self:CreateTooltip();

    self.vars.path = self:FindPath( start, goal );
    if( not self.vars.path ) then
        self.vars.path = self:FindPath( start, goal, true ); -- Try walking in opposing factions zones
    end--if
    if( not self.vars.path ) then
        tooltip:AddLine( "No path to destination found!" );
    else
        local zones = self.vars.zones;
        local showColoredText = self.db.profile.showColoredText;
        local pathSize = #self.vars.path;

        tooltip:AddLine( "Walk path" );
        for n, i in ipairs( self.vars.path ) do
            local text = self:GetPathText( zones[i], showColoredText );

            if( n ~= pathSize ) then
                tooltip:AddLine( text .. " ->" );
            else
                tooltip:AddLine( text );
            end--if
        end--for
    end--if
    self.vars.tooltip:Show();
end--Broker_RecZone:OnClickZone( zone )

function Broker_RecZone:CreateTooltip()
    local tooltip = self.vars.tooltip;
    local qtip = self.vars.qtip;

    if( not tooltip ) then
        tooltip = CreateFrame( "GameTooltip", "Broker_RecZoneTooltip", qtip, "GameTooltipTemplate" );
    end--if
    tooltip:SetOwner( qtip, "ANCHOR_NONE" );
    tooltip:ClearAllPoints();
    if( qtip:GetLeft() < GetScreenWidth() / 2 ) then
        tooltip:SetPoint( "LEFT", qtip, "RIGHT" );
    else
        tooltip:SetPoint( "RIGHT", qtip, "LEFT" );
    end--if
    self.vars.tooltip = tooltip;
    return tooltip;
end--Broker_RecZone:CreateTooltip()

function Broker_RecZone:FindPath( start, goal, opposing )
    local visited = {};
    local path = { start };
    local paths = { path };
    local zones = self.vars.zones;
    local function copytable( tbl, newitem )
        local newtbl = {};

        for _, a in ipairs( tbl ) do
            table.insert( newtbl, a );
        end--for
        table.insert( newtbl,newitem );
        return newtbl;
    end--function copytable( tbl,newitem )    

    visited[start] = true;
    while true do
        path = paths[1];

        local c = path[table.getn( path )];

        if( zones[c].zones ) then
            for _, d in ipairs( zones[c].zones ) do
                if not visited[d] then
                    local z = zones[d];

                    if( ( z.type == RZ_ZONE ) or ( z.faction == RZ_CONTESTED ) or ( z.faction == self.vars.playerFaction ) or opposing  ) then
                        visited[d]=true;

                        local newpath = copytable( path, d );

                        if( d == goal ) then
                            paths = nil;
                            visited = nil;
                            return newpath; -- Path found
                        end--if
                        table.insert( paths, newpath );
                    end--if
                end--if
            end--for
        end--if
        table.remove( paths, 1 );
        if( table.getn( paths ) == 0 ) then
            return; -- Nothing found
        end--if
    end--while
end--Broker_RecZone:FindPath( start, goal, opposing )

function Broker_RecZone:GetZoneNr( zone )
    local name = zone.name;
    local faction = zone.faction;

    for i, z in ipairs( self.vars.zones ) do
        if( z.name:find( name, 1, true ) and ( z.faction == faction ) ) then
            return i;
        end--if
    end--for
end--Broker_RecZone:GetZoneNr( zone )


-- Worldmap function --

function Broker_RecZone:ToggleWorldMapText()
    if( self.db.profile.showMap ) then
        Broker_RecZoneMapFrame:Show();
    else
        Broker_RecZoneMapFrame:Hide();
    end--if
end--Broker_RecZone:ToggleWorldMapText()

function Broker_RecZone.MapFrame_OnUpdate()
    local self = Broker_RecZone;
    local zoneText = "";

    if( self.db.profile.showMapCoords ) then
        local OFFSET_X = 0.0022;
        local OFFSET_Y = -0.0262;
        local x, y = GetCursorPosition();

        x = x / WorldMapFrame:GetScale();
        y = y / WorldMapFrame:GetScale();

        local px, py = GetPlayerMapPosition( "player" );
        local centerX, centerY = WorldMapFrame:GetCenter();
        local width = WorldMapButton:GetWidth();
        local height = WorldMapButton:GetHeight();
        local adjustedX = ( x - ( centerX - ( width / 2 ) ) ) / width;
        local adjustedY = ( centerY + ( height / 2 ) - y ) / height;
        local cx = adjustedX + OFFSET_X;
        local cy = adjustedY + OFFSET_Y;
        
        Broker_RecZoneMapFrameCursorCoords:SetText( ("%s %.0f, %.0f"):format( L["Cursor:"], 100 * cx, 100 * cy ) );
        Broker_RecZoneMapFramePlayerCoords:SetText( ("%s %.0f, %.0f"):format( L["Player:"], 100 * px, 100 * py ) );
    end--if

    if( self.db.profile.showMap ) then
        local areaName = WorldMapFrame.areaName;

        if( areaName ~= nil ) then
            local zones = self.vars.zones;
            local playerFaction = self.vars.playerFaction;

            for i, zone in ipairs( zones ) do
                if( areaName:find( zone.name, 1, true ) ) then
                    if( zone.type == RZ_PVP ) then
                        zoneText = ("%s(%d-%d %s)|r\n\n"):format( self:GetZoneColor( zone ), zone.low, zone.high, L["PVP"] );
                    else
                        zoneText = ("%s(%d-%d)|r\n\n"):format( self:GetZoneColor( zone ), zone.low, zone.high );
                    end--if
                    if( zone.zones ) then
                        local foundfirst = false;

                        for _, c in ipairs( zone.zones ) do
                            local cZones = zones[c];

                            if( ( cZones.type >= RZ_INSTANCE ) and
                                    ( ( cZones.faction == RZ_CONTESTED ) or ( cZones.faction == playerFaction ) ) ) then
                                if( not foundfirst ) then
                                    zoneText = zoneText .. HEX_YELLOW .. L["Instances:"] .. "|r\n";
                                    foundfirst = true;
                                end--if
                                zoneText = zoneText .. self:GetInstanceText( cZones, true ) .. "\n";
                            end--if
                        end--for
                    end--if
                end--if
            end--for
        end--if
        Broker_RecZoneMapText:SetText( zoneText );
    end--if
end--Broker_RecZone.MapFrame_OnUpdate()
